@@ -157,7 +157,7 @@ remove_locked_enuminfo(WsContextH cntx,
 		return;
 	}
 	hash_delete_free(cntx->enuminfos,
-	             hash_lookup(cntx->enuminfos, enumInfo->enumId));
+		hash_lookup(cntx->enuminfos, enumInfo->enumId));
 	u_unlock(cntx->soap);
 }
 
@@ -201,14 +201,22 @@ search_pull_subs_info(SoapH soap, WsXmlDocH indoc)
 		uuid = ws_xml_get_node_text(node);
 	}
 	if(uuid == NULL) return subsInfo;
-	pthread_mutex_lock(&soap->lockSubs);
+	if (pthread_mutex_lock(&soap->lockSubs))
+	{
+		error("Error: Can't lock soap->lockSubs");
+		return NULL;
+	}
 	lnode = list_first(soapCntx->subscriptionMemList);
 	while(lnode) {
 		subsInfo = (WsSubscribeInfo *)lnode->list_data;
 		if(!strcmp(subsInfo->subsId, uuid+5)) break;
 		lnode = list_next(soapCntx->subscriptionMemList, lnode);
 	}
-	pthread_mutex_unlock(&soap->lockSubs);
+	if (pthread_mutex_unlock(&soap->lockSubs))
+	{
+		error("Error: Can't unlock soap->lockSubs");
+		return NULL;
+	}
 	if(lnode == NULL) return NULL;
 	return subsInfo;
 }
@@ -247,8 +255,18 @@ create_enum_info(SoapOpH op,
 	enumInfo->releaseproc = wsman_get_release_endpoint(epcntx, indoc);
 	to = ws_xml_get_node_text(
 			ws_xml_get_child(header, 0, XML_NS_ADDRESSING, WSA_TO));
+	if (to == NULL) {
+		error("No to node");
+		fault_code = WSMAN_INTERNAL_ERROR;
+		goto DONE;
+	}
 	uri =  ws_xml_get_node_text(
 			ws_xml_get_child(header, 0, XML_NS_WS_MAN, WSM_RESOURCE_URI));
+	if (uri == NULL) {
+		error("No uri node");
+		fault_code = WSMAN_INTERNAL_ERROR;
+		goto DONE;
+	}
 
 	enumInfo->epr_to = u_strdup(to);
 	enumInfo->epr_uri = u_strdup(uri);
@@ -280,6 +298,7 @@ DONE:
 	if (fault_code != WSMAN_RC_OK) {
 		outdoc = wsman_generate_fault(indoc, fault_code, fault_detail_code, NULL);
 		u_free(enumInfo);
+                *eInfo = NULL;
 	} else {
 		*eInfo = enumInfo;
 	}
@@ -314,6 +333,12 @@ wsman_verify_enum_info(SoapOpH op,
 			XML_NS_ADDRESSING, WSA_TO));
 	char *uri= ws_xml_get_node_text(ws_xml_get_child(header, 0,
 			XML_NS_WS_MAN, WSM_RESOURCE_URI));
+	if (to == NULL || uri == NULL) {
+		error("No to or uri node");
+		status->fault_code = WSMAN_INTERNAL_ERROR;
+		status->fault_detail_code = 0;
+		return 0;
+	}
 
 	if (strcmp(enumInfo->epr_to, to) != 0 ||
 			strcmp(enumInfo->epr_uri, uri) != 0 ) {
@@ -459,10 +484,12 @@ make_callback_entry(SoapServiceCallback proc,
 	if (entry) {
 		lnode_init(&entry->node, data);
 		entry->proc = proc;
-		if (list_to_add == NULL) {
-			list_to_add = list_create(LISTCOUNT_T_MAX);
+		if (list_to_add || (list_to_add = list_create(LISTCOUNT_T_MAX))) {
+			list_append(list_to_add, &entry->node);
+		} else {
+			u_free(entry);
+			return NULL;
 		}
-		list_append(list_to_add, &entry->node);
 	} else {
 		return NULL;
 	}
@@ -475,11 +502,15 @@ void
 ws_initialize_context(WsContextH cntx, SoapH soap)
 {
 	cntx->entries = hash_create(HASHCOUNT_T_MAX, NULL, NULL);
-	hash_set_allocator(cntx->entries, NULL, free_hentry_func, NULL);
+	if (cntx->entries) {
+		hash_set_allocator(cntx->entries, NULL, free_hentry_func, NULL);
+	}
 
 	cntx->enuminfos = hash_create(HASHCOUNT_T_MAX, NULL, NULL);
 	cntx->subscriptionMemList = list_create(LISTCOUNT_T_MAX);
-	hash_set_allocator(cntx->enuminfos, NULL, free_hentry_func, NULL);
+	if (cntx->enuminfos) {
+		hash_set_allocator(cntx->enuminfos, NULL, free_hentry_func, NULL);
+	}
 	cntx->owner = 1;
 	cntx->soap = soap;
 	cntx->serializercntx = ws_serializer_init();
@@ -549,7 +580,7 @@ ws_create_runtime(list_t * interfaces)
 	dispInfo = (WsManDispatcherInfo *) u_zalloc(size);
 	if (dispInfo == NULL) {
 		error("Could not allocate memory");
-		u_free(soap);
+		soap_destroy(soap);
 		return NULL;
 	}
 	debug("Registering %d plugins", (int) list_count(interfaces));
@@ -745,9 +776,19 @@ wsman_identify_stub(SoapOpH op,
 	SoapH           soap;
 	WsEndPointGet   endPoint;
 
-	status = u_zalloc(sizeof(WsmanStatus *));
 	soap = soap_get_op_soap(op);
+	if (soap == NULL) {
+		error("soap_get_op_soap failed");
+		return -1;
+	}
+
+	status = u_zalloc(sizeof(WsmanStatus *));
 	cntx = ws_create_ep_context(soap, soap_get_op_doc(op, 1));
+	if (cntx == NULL) {
+		error("ws_create_ep_context failed");
+		u_free(status);
+		return -1;
+	}
 	info = (WsDispatchEndPointInfo *) appData;
 	typeInfo = info->serializationInfo;
 	endPoint = (WsEndPointGet) info->serviceEndPoint;
@@ -789,7 +830,17 @@ ws_transfer_put_stub(SoapOpH op,
 	void           *outData = NULL;
 	WsmanStatus     status;
 	SoapH           soap = soap_get_op_soap(op);
+
+	if (soap == NULL) {
+		error("soap_get_op_soap failed");
+		return -1;
+	}
+
 	WsContextH   cntx = ws_create_ep_context(soap, soap_get_op_doc(op, 1));
+	if (cntx == NULL) {
+		error("ws_create_ep_context");
+		return -1;
+	}
 	WsDispatchEndPointInfo *info = (WsDispatchEndPointInfo *) appData;
 	XmlSerializerInfo *typeInfo = info->serializationInfo;
 	WsEndPointPut   endPoint = (WsEndPointPut) info->serviceEndPoint;
@@ -833,6 +884,12 @@ ws_transfer_delete_stub(SoapOpH op,
 {
 	WsmanStatus     status;
 	SoapH           soap = soap_get_op_soap(op);
+
+	if (soap == NULL) {
+		error("soap_get_op_soap failed");
+		return -1;
+	}
+
 	WsContextH      cntx = ws_create_ep_context(soap,
 					soap_get_op_doc(op, 1));
 
@@ -844,7 +901,7 @@ ws_transfer_delete_stub(SoapOpH op,
 
 	wsman_status_init(&status);
 	if ((data = endPoint(cntx, &status, opaqueData)) == NULL) {
-		warning("Transfer Delete fault");
+		_warning("Transfer Delete fault");
 		doc = wsman_generate_fault(soap_get_op_doc(op, 1),
 					 WSMAN_INVALID_SELECTORS, 0, NULL);
 	} else {
@@ -873,8 +930,17 @@ ws_transfer_get_stub(SoapOpH op,
 
 
 	SoapH       soap = soap_get_op_soap(op);
-	WsContextH  cntx = ws_create_ep_context(soap, soap_get_op_doc(op, 1));
 
+	if (soap == NULL) {
+		error("soap_get_op_soap failed");
+		return -1;
+	}
+
+	WsContextH  cntx = ws_create_ep_context(soap, soap_get_op_doc(op, 1));
+	if (cntx == NULL) {
+		error("ws_create_ep_context failed");
+		return -1;
+	}
 	WsDispatchEndPointInfo *info = (WsDispatchEndPointInfo *) appData;
 	XmlSerializerInfo *typeInfo = info->serializationInfo;
 	WsEndPointGet   endPoint = (WsEndPointGet) info->serviceEndPoint;
@@ -884,7 +950,7 @@ ws_transfer_get_stub(SoapOpH op,
 
 	wsman_status_init(&status);
 	if ((data = endPoint(cntx, &status, opaqueData)) == NULL) {
-		warning("Transfer Get fault");
+		_warning("Transfer Get fault");
 		doc = wsman_generate_fault( soap_get_op_doc(op, 1),
 					 WSMAN_INVALID_SELECTORS, 0, NULL);
 	} else {
@@ -900,7 +966,7 @@ ws_transfer_get_stub(SoapOpH op,
 		debug("Setting operation document");
 		soap_set_op_doc(op, doc, 0);
 	} else {
-		warning("Response doc invalid");
+		_warning("Response doc invalid");
 	}
 
 	ws_destroy_context(cntx);
@@ -944,16 +1010,18 @@ unsigned long get_total_enum_context(WsContextH cntx){
  * preset, hence marking them as weak symbols and testing to see
  * if they are resolved before using them.
  */
-#ifndef __CYGWIN__
+#if !defined(_MSC_VER) || !defined(__CYGWIN__)
 #pragma weak wsmand_options_get_max_threads
+#endif /* _MSC_VER */
 extern int wsmand_options_get_max_threads(void);
+#if !defined(_MSC_VER) || !defined(__CYGWIN__)
 #pragma weak wsmand_options_get_max_connections_per_thread
+#endif /* _MSC_VER */
 extern int wsmand_options_get_max_connections_per_thread(void);
-#endif
 
 /**
  * Enumeration Stub for processing enumeration requests
- * @param op SOAP pperation handler
+ * @param op SOAP operation handler
  * @param appData Application data
  * @return status
  */
@@ -970,6 +1038,11 @@ wsenum_enumerate_stub(SoapOpH op,
 	WsContextH      soapCntx;
 	SoapH           soap = soap_get_op_soap(op);
 
+	if (soap == NULL) {
+		error("soap_get_op_soap failed");
+		return -1;
+	}
+
 	WsDispatchEndPointInfo *ep = (WsDispatchEndPointInfo *) appData;
 	WsEndPointEnumerate endPoint =
 			(WsEndPointEnumerate)ep->serviceEndPoint;
@@ -1018,8 +1091,12 @@ wsenum_enumerate_stub(SoapOpH op,
                 }
         }
 
-	epcntx = ws_create_ep_context(soap, _doc);
 	wsman_status_init(&status);
+
+	epcntx = ws_create_ep_context(soap, _doc);
+	if (epcntx == NULL) {
+		goto DONE;
+	}
 	doc = create_enum_info(op, epcntx, _doc, &enumInfo);
 	if (doc != NULL) {
 		/* wrong enum elements met. Fault message generated */
@@ -1030,7 +1107,6 @@ wsenum_enumerate_stub(SoapOpH op,
                 debug("enumeration fault");
 		doc = wsman_generate_fault( _doc, status.fault_code,
 				status.fault_detail_code, status.fault_msg);
-		destroy_enuminfo(enumInfo);
 		goto DONE;
 	}
 	if (enumInfo->pullResultPtr) {
@@ -1045,6 +1121,9 @@ wsenum_enumerate_stub(SoapOpH op,
 
 	wsman_set_estimated_total(_doc, doc, enumInfo);
 	body = ws_xml_get_soap_body(doc);
+	if (!body) {
+		goto DONE;
+	}
 
 	if (enumInfo->pullResultPtr == NULL) {
 		resp_node = ws_xml_add_child(body, XML_NS_ENUMERATION,
@@ -1053,6 +1132,9 @@ wsenum_enumerate_stub(SoapOpH op,
 		resp_node = ws_xml_get_child(body, 0,
 				 XML_NS_ENUMERATION, WSENUM_ENUMERATE_RESP);
 	}
+	if (!resp_node) {
+		goto DONE;
+	}
 
 	soapCntx = ws_get_soap_context(soap);
 	if (( enumInfo->flags & WSMAN_ENUMINFO_OPT ) == WSMAN_ENUMINFO_OPT  &&
@@ -1061,11 +1143,11 @@ wsenum_enumerate_stub(SoapOpH op,
 			    XML_NS_ENUMERATION, WSENUM_ENUMERATION_CONTEXT, 0);
 		ws_serialize_str(epcntx->serializercntx, resp_node,
 			       NULL, XML_NS_WS_MAN, WSENUM_END_OF_SEQUENCE, 0);
-		destroy_enuminfo(enumInfo);
 	} else {
 		ws_serialize_str(epcntx->serializercntx, resp_node, enumInfo->enumId,
 			    XML_NS_ENUMERATION, WSENUM_ENUMERATION_CONTEXT, 0);
 		insert_enum_info(soapCntx, enumInfo);
+		enumInfo = NULL;
 	}
 
 DONE:
@@ -1074,6 +1156,9 @@ DONE:
 	}
 	ws_destroy_context(epcntx);
 	u_free(status.fault_msg);
+	if (enumInfo) {
+		destroy_enuminfo(enumInfo);
+	}
 	return retVal;
 }
 
@@ -1088,6 +1173,12 @@ wsenum_release_stub(SoapOpH op,
 	WsXmlDocH       doc = NULL;
 	WsmanStatus     status;
 	SoapH           soap = soap_get_op_soap(op);
+
+	if (soap == NULL) {
+		error("soap_get_op_soap failed");
+		return -1;
+	}
+
 	WsContextH      soapCntx = ws_get_soap_context(soap);
 	WsDispatchEndPointInfo *ep = (WsDispatchEndPointInfo *) appData;
 	WsEndPointRelease endPoint = (WsEndPointRelease) ep->serviceEndPoint;
@@ -1136,6 +1227,12 @@ wsenum_pull_stub(SoapOpH op, void *appData,
 	WsXmlNodeH      node;
 	WsmanStatus     status;
 	SoapH           soap = soap_get_op_soap(op);
+
+	if (soap == NULL) {
+		error("soap_get_op_soap failed");
+		return -1;
+	}
+
 	WsContextH      soapCntx = ws_get_soap_context(soap);
 
 	WsDispatchEndPointInfo *ep = (WsDispatchEndPointInfo *) appData;
@@ -1148,6 +1245,7 @@ wsenum_pull_stub(SoapOpH op, void *appData,
 
 	WsXmlDocH _doc = soap_get_op_doc(op, 1);
 	WsEnumerateInfo *enumInfo;
+	WsXmlNodeH body;
 
 	wsman_status_init(&status);
 	enumInfo = get_locked_enuminfo(soapCntx, _doc,
@@ -1171,9 +1269,12 @@ wsenum_pull_stub(SoapOpH op, void *appData,
 	}
 
 	wsman_set_estimated_total(_doc, doc, enumInfo);
-	node = ws_xml_add_child(ws_xml_get_soap_body(doc),
-				XML_NS_ENUMERATION, WSENUM_PULL_RESP, NULL);
+	body = ws_xml_get_soap_body(doc);
+	if (!body) {
+		goto DONE;
+	}
 
+	node = ws_xml_add_child(body, XML_NS_ENUMERATION, WSENUM_PULL_RESP, NULL);
 	if (node == NULL) {
 		goto DONE;
 	}
@@ -1219,6 +1320,12 @@ wsenum_pull_direct_stub(SoapOpH op,
 	WsmanStatus     status;
 	WsXmlDocH       doc = NULL;
 	SoapH           soap = soap_get_op_soap(op);
+
+	if (soap == NULL) {
+		error("soap_get_op_soap failed");
+		return -1;
+	}
+
 	WsContextH      soapCntx = ws_get_soap_context(soap);
 	WsDispatchEndPointInfo *ep = (WsDispatchEndPointInfo *) appData;
 #ifdef ENABLE_EVENTING_SUPPORT
@@ -1265,7 +1372,11 @@ wsenum_pull_direct_stub(SoapOpH op,
 					XML_NS_ENUMERATION, WSENUM_PULL_RESP);
 			items = ws_xml_get_child(response, 0,
                                         XML_NS_ENUMERATION, WSENUM_ITEMS);
-
+			if (items == NULL) {
+				error("Invalid enumeration items...");
+				doc = wsman_generate_fault( _doc, WSMAN_INTERNAL_ERROR, WSMAN_DETAIL_INVALID, NULL);
+				goto cleanup;
+			}
 			if (enumInfo->totalItems == 0 || enumInfo->index == enumInfo->totalItems) {
 				/*
 				   ws_serialize_str(soapCntx, response, NULL,
@@ -1544,7 +1655,7 @@ create_subs_info(SoapOpH op,
 	}
 	subsInfo->uri = u_strdup(wsman_get_resource_uri(epcntx, indoc));
 	if(!subNode) {
-		message("No subsribe body");
+		message("No subscribe body");
 		fault_code = WSE_INVALID_MESSAGE;
 		goto DONE;
 	}
@@ -1703,7 +1814,7 @@ create_subs_info(SoapOpH op,
 	temp = ws_xml_get_soap_header(indoc);
 	temp = ws_xml_get_child(temp, 0, XML_NS_OPENWSMAN, "FormerUID");
 	ntext = ws_xml_get_node_text(temp);
-	if(temp && ntext) { //it is a request from the saved reqeust. So we recover the former UUID
+	if(temp && ntext) { //it is a request from the saved request. So we recover the former UUID
 		strncpy(subsInfo->subsId, ntext, EUIDLEN);
 		debug("Recover to uuid:%s",subsInfo->subsId);
 	}
@@ -1724,7 +1835,7 @@ DONE:
 
 /**
  * Subscribe Stub for processing subscription requests
- * @param op SOAP pperation handler
+ * @param op SOAP operation handler
  * @param appData Application data
  * @return status
  */
@@ -1737,6 +1848,12 @@ wse_subscribe_stub(SoapOpH op, void *appData, void *opaqueData)
 	WsmanStatus     status;
 	WsXmlNodeH      inNode, body, header, temp;
 	SoapH           soap = soap_get_op_soap(op);
+
+	if (soap == NULL) {
+		error("soap_get_op_soap failed");
+		return -1;
+	}
+
 	WsContextH soapCntx = ws_get_soap_context(soap);
 	int i;
 	WsDispatchEndPointInfo *ep = (WsDispatchEndPointInfo *) appData;
@@ -1835,7 +1952,7 @@ DONE:
 
 /**
  * Unsubscribe Stub for processing unsubscription requests
- * @param op SOAP pperation handler
+ * @param op SOAP operation handler
  * @param appData Application data
  * @return status
  */
@@ -1849,6 +1966,12 @@ wse_unsubscribe_stub(SoapOpH op, void *appData, void *opaqueData)
 	WsXmlNodeH      inNode;
 	WsXmlNodeH      header;
 	SoapH           soap = soap_get_op_soap(op);
+
+	if (soap == NULL) {
+		error("soap_get_op_soap failed");
+		return -1;
+	}
+
 	WsContextH soapCntx = ws_get_soap_context(soap);
 	WsDispatchEndPointInfo *ep = (WsDispatchEndPointInfo *) appData;
 	WsEndPointSubscribe endPoint =
@@ -1912,7 +2035,7 @@ DONE:
 
 /**
  * Renew Stub for processing renew requests
- * @param op SOAP pperation handler
+ * @param op SOAP operation handler
  * @param appData Application data
  * @return status
  */
@@ -1927,6 +2050,12 @@ wse_renew_stub(SoapOpH op, void *appData, void *opaqueData)
 	WsXmlNodeH      body;
 	WsXmlNodeH      header;
 	SoapH           soap = soap_get_op_soap(op);
+
+	if (soap == NULL) {
+		error("soap_get_op_soap failed");
+		return -1;
+	}
+
 	WsContextH soapCntx = ws_get_soap_context(soap);
 	char * expirestr = NULL;
 
@@ -2119,7 +2248,7 @@ static int wse_send_notification(WsEventThreadContextH cntx, WsXmlDocH outdoc, W
 	}
 	wsmc_transport_init(notificationSender, NULL);
 	if (wsman_send_request(notificationSender, outdoc)) {
-                warning("wse_send_notification: wsman_send_request fails for endpoint %s", subsInfo->epr_notifyto);
+                _warning("wse_send_notification: wsman_send_request fails for endpoint %s", subsInfo->epr_notifyto);
                 /* FIXME: retVal */
         }
 	if(acked) {
@@ -2416,6 +2545,11 @@ create_context_entry(hash_t * h,
 {
 	char           *key = u_strdup(name);
 	hnode_t        *hn = hnode_create(val);
+	if (!key || !hn) {
+		u_free(key);
+		u_free(hn);
+		return NULL;
+	}
 	hash_insert(h, hn, (void *) key);
 	return hn;
 }
